/************************************************************************
 *   IRC - Internet Relay Chat, src/services.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>

#include "m_commands.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "send.h"
#include "hash.h"
#include "channel.h"
#include "irc_string.h"
#include "s_conf.h"
#include "config.h"

/* 
 * This is code for services support (/nickserv, /chanserv etc).
 * All pretty basic..
 */

/* deliver_services_message - deliver a message destined for the named service */

#ifdef SERVICES
int
deliver_services_message(char *service, char *command,
                         struct Client *cptr, struct Client *sptr, int parc, char **parv)
{
	struct Client *acptr;
	char arg[BUFSIZE + 1];
	int i;

	if (!MyClient(sptr) || IsServer(cptr)) /* we don't support routing /nickserv *yet* */
		return 0;

	/* check the host */
	if (!ConfigFileEntry.serviceshost)
	{
		/* no S: line duh. */
		sendto_one(sptr, ":%s NOTICE %s :Sorry, cannot process your command because services host is not defined in configuraton file",
		           me.name, parv[0]);
		return 0;
	}

	if (parc < 2) 
	{
		sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], command);
		return 0;
	}

	/* first of all, make sure the nick is on line */
	acptr = find_client(service, NULL);
	if (!acptr || IsServer(acptr))
	{
		sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, parv[0], service);
		return 0;
	}

	if (strcasecmp(acptr->user->server, ConfigFileEntry.serviceshost))
	{
		sendto_one(sptr, ":%s NOTICE %s :Message not delivered: server mismatch for %s (%s != %s)",
		           me.name, parv[0], service, acptr->user->server, ConfigFileEntry.serviceshost);
		return 0;
	}

	/* concatenate each argument together.  I'm not sure if this is a good idea, but Unreal
	   implements it, and users appear to expect it... */
	memset(arg, 0, sizeof(arg));
	for (i = 1; i < parc; i++) 
	{
		if (strlen(arg) + strlen(parv[i]) > BUFSIZE - 2)
			break;
		strcat(arg, parv[i]);
		strcat(arg, " ");
	}

	/* and finally, deliver the message.. */
	sendto_one(acptr, ":%s PRIVMSG %s :%s", sptr->name, acptr->name, arg);
	return 0;
}

#define DEFUN_SERV(a,b,c) int a (struct Client *cptr, struct Client *sptr, int parc, char **parv) { return deliver_services_message (b, c, cptr,sptr,parc,parv); }

#ifdef SERVICES_SHORT_ALIASES
#define DEFUN_ALIAS(a,b,c) int a (struct Client *cptr, struct Client *sptr, int parc, char **parv) { return b (cptr,sptr,parc,parv); }
#else
#define DEFUN_ALIAS(a,b,c) int a (struct Client *cptr, struct Client *sptr, int parc, char **parv) { \
				sendto_one(sptr, ":%s NOTICE %s :Please use /%s", me.name, parv[0], c); \
				return 0; \
				}
#endif

#ifdef NICKSERV
DEFUN_SERV(m_nickserv, NICKSERV, "NICKSERV")
DEFUN_ALIAS(m_ns, m_nickserv, "nickserv")
#endif

#ifdef CHANSERV
DEFUN_SERV(m_chanserv, CHANSERV, "CHANSERV")
DEFUN_ALIAS(m_cs, m_chanserv, "chanserv")
#endif

#ifdef OPERSERV
DEFUN_SERV(m_operserv, OPERSERV, "OPERSERV")
DEFUN_ALIAS(m_os, m_operserv, "operserv")
#endif

#ifdef MEMOSERV
DEFUN_SERV(m_memoserv, MEMOSERV, "MEMOSERV")
DEFUN_ALIAS(m_ms, m_memoserv, "memoserv")
#endif

#ifdef STATSERV
DEFUN_SERV(m_statserv, STATSERV, "STATSERV")
DEFUN_ALIAS(m_ss, m_statserv, "statserv")
#endif

#ifdef VOTESERV
DEFUN_SERV(m_voteserv, VOTESERV, "VOTESERV")
DEFUN_ALIAS(m_vs, m_voteserv, "voteserv")
#endif

#ifdef SEENSERV
DEFUN_SERV(m_seenserv, SEENSERV, "SEENSERV")
#endif

#endif
